# -*- coding: utf-8 -*-

import time
import configparser

import pymysql
from dbutils.pooled_db import PooledDB

from ydl_ai_recommender.src.utils.log import create_logger


class MySQLClient():

    def __init__(self, host, port, user, password) -> None:
        self.logger = create_logger(__name__, 'mysql_client.log', is_rotating=True)
        try:
            self.connection = pymysql.connect(
                host=host,
                port=port,
                user=user,
                password=password,
                charset='utf8mb4',
                cursorclass=pymysql.cursors.DictCursor
            )
        except Exception as e:
            self.logger.error('数据库连接失败', exc_info=True)
            raise e
        self.cursor = self.connection.cursor()
        self.logger.info('数据库连接成功')


    def query(self, sql):
        # sql += ' limit 1000'
        st = time.time()
        self.logger.debug('begin execute sql: %s', sql)
        row_count = self.cursor.execute(sql)
        data = self.cursor.fetchall()
        cost_time = time.time() - st
        self.logger.debug('fetch row count: %s, cost time %s', row_count, cost_time)
        return row_count, data


    def __del__(self):
        try:
            self.cursor.close()
            self.connection.close()
            # 容易触发 NameError: name 'open' is not defined
            # self.logger.info('dataset disconnected')
        except Exception as e:
            self.logger.error('销毁 MySQLClient 失败', exc_info=True)
            print(e)


    @classmethod
    def create_from_config_file(cls, config_file, section='ADB'):
        config = configparser.RawConfigParser()
        config.read(config_file)
        return cls(
            config.get(section, 'host'),
            config.getint(section, 'port'),
            config.get(section, 'user'),
            config.get(section, 'password')
        )



class MySQLClientPool():

    def __init__(self, host, port, user, password) -> None:
        self.logger = create_logger(__name__, 'mysql_client.log', is_rotating=True)
        try:
            self.pool = PooledDB(
                creator=pymysql,
                maxconnections=8,  # 连接池允许的最大连接数，0和None表示不限制连接数
                mincached=2,  # 初始化时，链接池中至少创建的空闲的链接，0表示不创建
                blocking=True,  # 连接池中如果没有可用连接后，是否阻塞等待。True，等待；False，不等待然后报错
                ping=0,  # ping MySQL服务端，检查是否服务可用。如：0 = None = never, 1 = default = whenever it is requested, 2 = when a cursor is created, 4 = when a query is executed, 7 = always
                host=host,
                port=port,
                user=user,
                password=password,
                charset='utf8mb4',
                cursorclass=pymysql.cursors.DictCursor
            )
        except Exception as e:
            self.logger.error('数据库连接失败', exc_info=True)
            raise e

        self.logger.info('数据库连接成功')

    def open(self):
        conn = self.pool.connection()  # 去连接池中获取一个连接
        cur = conn.cursor()
        return conn, cur

    def close(self, conn, cur):
        cur.close()
        conn.close()  # 将连接放回到连接池，并不会关闭连接，当线程终止时，连接自动关闭

    def query(self, sql, args=None):
        # sql += ' limit 1000'
        st = time.time()
        self.logger.debug('begin execute sql: %s', sql)
        conn, cur = self.open()
        row_count = cur.execute(sql, args)
        data = cur.fetchall()
        self.close(conn, cur)
        cost_time = time.time() - st
        self.logger.debug('fetch row count: %s, cost time %s', row_count, cost_time)
        return row_count, data

    @classmethod
    def create_from_config_file(cls, config_file, section='ADB'):
        config = configparser.RawConfigParser()
        config.read(config_file)
        return cls(
            config.get(section, 'host'),
            config.getint(section, 'port'),
            config.get(section, 'user'),
            config.get(section, 'password')
        )